@@ -6,7 +6,6 @@ import ConfigParser
 import subprocess
 import tempfile
 import stdeb
-import pkg_resources
 from stdeb import log, __version__ as __stdeb_version__
 
 if hasattr(os,'link'):
@@ -30,6 +29,7 @@ PYSUPPORT_MIN_VERS = '0.8.4' # Namespace package support was added
 
 import exceptions
 class CalledProcessError(exceptions.Exception): pass
+class CantSatisfyRequirement(exceptions.Exception): pass
 
 def check_call(*popenargs, **kwargs):
     retcode = subprocess.call(*popenargs, **kwargs)
@@ -42,13 +42,17 @@ stdeb_cmdline_opts = [
      "directory to put final built distributions in (default='deb_dist')"),
     ('patch-already-applied','a',
      'patch was already applied (used when py2dsc calls sdist_dsc)'),
-    ('default-distribution=', 'z',
+    ('default-distribution=', None,
+     "deprecated (see --suite)"),
+    ('suite=', 'z',
      "distribution name to use if not specified in .cfg (default='unstable')"),
-    ('default-maintainer=', 'm',
+    ('default-maintainer=', None,
+     'deprecated (see --maintainer)'),
+    ('maintainer=', 'm',
      'maintainer name and email to use if not specified in .cfg '
      '(default from setup.py)'),
     ('extra-cfg-file=','x',
-     'additional .cfg file (in addition to .egg-info/stdeb.cfg if present)'),
+     'additional .cfg file (in addition to stdeb.cfg if present)'),
     ('patch-file=','p',
      'patch file applied before setup.py called '
      '(incompatible with file specified in .cfg)'),
@@ -166,7 +170,6 @@ def recursive_hardlink(src,dst):
 def debianize_name(name):
     "make name acceptable as a Debian (binary) package name"
     name = name.replace('_','')
-    name = name.replace('.','-')
     name = name.lower()
     return name
 
@@ -237,7 +240,19 @@ def load_module(name,fname):
         fd.close()
     return module
 
-def get_deb_depends_from_setuptools_requires(requirements):
+def get_deb_depends_from_setuptools_requires(requirements, on_failure="warn"):
+    """
+    Suppose you can't confidently figure out a .deb which satisfies a given
+    requirement.  If on_failure == 'warn', then log a warning.  If on_failure
+    == 'raise' then raise CantSatisfyRequirement exception.  If on_failure ==
+    'guess' then guess that python-$FOO will satisfy the dependency and that
+    the Python version numbers will apply to the Debian packages (in addition
+    to logging a warning message).
+    """
+    assert on_failure in ("raise", "warn", "guess"), on_failure
+
+    import pkg_resources
+
     depends = [] # This will be the return value from this function.
 
     parsed_reqs=[]
@@ -261,9 +276,20 @@ def get_deb_depends_from_setuptools_requires(requirements):
     # "foo.egg-info" directory.
 
     egginfore=("(/(%s)(?:-[^/]+)?(?:-py[0-9]\.[0-9.]+)?\.egg-info)"
-               % '|'.join(req.project_name for req in parsed_reqs))
+               % '|'.join(req.project_name.replace('-', '_') for req in parsed_reqs))
 
     args = ["apt-file", "search", "--ignore-case", "--regexp", egginfore]
+
+    if 1:
+        # do dry run on apt-file
+        dry_run_args = args[:] + ['--dummy','--non-interactive']
+        cmd = subprocess.Popen(dry_run_args,stderr=subprocess.PIPE)
+        returncode = cmd.wait()
+        if returncode:
+            err_output = cmd.stderr.read()
+            raise RuntimeError('Error running "apt-file search": ' +
+                               err_output.strip())
+
     try:
         cmd = subprocess.Popen(args, stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
@@ -325,9 +351,25 @@ def get_deb_depends_from_setuptools_requires(requirements):
                          "\"%s\" -- ignoring."
                          % (', '.join(debs), req.project_name, req))
         if not gooddebs:
-            log.warn("I found no Debian package which provides the required "
-                     "Python package \"%s\" with version requirements "
-                     "\"%s\"."% (req.project_name, req.specs))
+            if on_failure == 'warn':
+                log.warn(
+                    "I found no Debian package which provides the required "
+                    "Python package \"%s\" with version requirements "
+                    "\"%s\"."% (req.project_name, req.specs))
+            elif on_failure == "raise":
+                raise CantSatisfyRequirement(
+                    "I found no Debian package which "
+                    "provides the required Python package \"%s\" with version "
+                    "requirements \"%s\"." % (req.project_name, req.specs), req)
+            elif on_failure == "guess":
+                log.warn("I found no Debian package which provides the "
+                         "required Python package \"%s\" with version "
+                         "requirements \"%s\".  Guessing blindly that the "
+                         "name \"python-%s\" will be it, and that the Python "
+                         "package version number requirements will apply to "
+                         "the Debian package." % (req.project_name,
+                                                  req.specs, reqname))
+                gooddebs.add("python-" + reqname)
         elif len(gooddebs) == 1:
             log.info("I found a Debian package which provides the require "
                      "Python package.  Python package: \"%s\", "
@@ -522,11 +564,14 @@ def apt_cache_info(apt_cache_cmd,package_name):
     args = ["apt-cache", apt_cache_cmd, package_name]
     cmd = subprocess.Popen(args,
                            stdin=subprocess.PIPE,
+                           stderr=subprocess.PIPE,
                            stdout=subprocess.PIPE)
     returncode = cmd.wait()
     if returncode:
-        log.error('ERROR running: %s', ' '.join(args))
-        raise RuntimeError('returncode %d from subprocess %s' % (returncode,
+        errline = cmd.stderr.read()
+        if not (returncode == 100 and errline == "E: You must put some 'source' URIs in your sources.list\n"):
+            log.error('ERROR running: %s', ' '.join(args))
+            raise RuntimeError('returncode %d from subprocess %s' % (returncode,
                                                                  args))
     inlines = cmd.stdout.read()
     version_blocks = inlines.split('\n\n')
@@ -557,6 +602,27 @@ def apt_cache_info(apt_cache_cmd,package_name):
         result_list.append(block_dict)
     return result_list
 
+def check_cfg_files(cfg_files,module_name):
+    """check if the configuration files actually specify something
+
+    If config files are given, give warning if they don't contain
+    information. This may indicate a wrong module name name, for
+    example.
+    """
+
+    cfg = ConfigParser.SafeConfigParser()
+    cfg.read(cfg_files)
+    if cfg.has_section(module_name):
+        section_items = cfg.items(module_name)
+    else:
+        section_items = []
+    default_items = cfg.items('DEFAULT')
+
+    n_items = len(section_items) + len(default_items)
+    if n_items==0:
+        log.warn('configuration files were specified, but no options were '
+                 'found in "%s" or "DEFAULT" sections.' % (module_name,) )
+
 class DebianInfo:
     """encapsulate information for Debian distribution system"""
     def __init__(self,
@@ -603,6 +669,9 @@ class DebianInfo:
             guess_maintainer=guess_maintainer,
             )
 
+        if len(cfg_files):
+            check_cfg_files(cfg_files,module_name)
+
         cfg = ConfigParser.SafeConfigParser(cfg_defaults)
         cfg.read(cfg_files)
 
@@ -614,6 +683,8 @@ class DebianInfo:
                 name = name.replace('-','_')
                 value = getattr( sdist_dsc_command, name )
                 if value is not None:
+                    if not cfg.has_section(module_name):
+                        cfg.add_section(module_name)
                     cfg.set( module_name, name, value )
 
         self.stdeb_version = __stdeb_version__
@@ -660,10 +731,11 @@ class DebianInfo:
         build_deps = []
         if use_setuptools:
             build_deps.append('python-setuptools (>= 0.6b3)')
-        build_deps.extend(
-            get_deb_depends_from_setuptools_requires(setup_requires))
+	if setup_requires is not None and len(setup_requires):
+            build_deps.extend(
+                get_deb_depends_from_setuptools_requires(setup_requires))
 
-        depends = ['${python:Depends}']
+        depends = ['${misc:Depends}', '${python:Depends}']
         need_custom_binary_target = False
 
         self.do_pycentral_removal_preinst = pycentral_backwards_compatibility
@@ -681,25 +753,17 @@ class DebianInfo:
         self.shared_mime_file = parse_val(cfg,module_name,'Shared-MIME-File')
 
         if self.mime_file == '' and self.shared_mime_file == '':
-            self.dh_installmime_line = ''
+            self.dh_installmime_indep_line = ''
         else:
             need_custom_binary_target = True
-            self.dh_installmime_line = '\tdh_installmime'
-            if self.architecture == 'all':
-                self.dh_installmime_line += ' -i'
-            else:
-                self.dh_installmime_line += ' -a'
+            self.dh_installmime_indep_line = '\tdh_installmime'
 
         mime_desktop_files = parse_vals(cfg,module_name,'MIME-Desktop-Files')
         if len(mime_desktop_files):
             need_custom_binary_target = True
-            self.dh_desktop_line = '\tdh_desktop'
-            if self.architecture == 'all':
-                self.dh_desktop_line += ' -i'
-            else:
-                self.dh_desktop_line += ' -a'
+            self.dh_desktop_indep_line = '\tdh_desktop'
         else:
-            self.dh_desktop_line = ''
+            self.dh_desktop_indep_line = ''
 
         #    E. any mime .desktop files
         self.install_file_lines = []
@@ -708,8 +772,9 @@ class DebianInfo:
                 '%s usr/share/applications'%mime_desktop_file)
 
         depends.extend(parse_vals(cfg,module_name,'Depends') )
-        depends.extend(get_deb_depends_from_setuptools_requires(
-            install_requires))
+	if install_requires is not None and len(install_requires):
+            depends.extend(get_deb_depends_from_setuptools_requires(
+                install_requires))
         self.depends = ', '.join(depends)
 
         self.debian_section = parse_val(cfg,module_name,'Section')
@@ -833,12 +898,12 @@ XB-Python-Version: ${python:Versions}
             cfg,module_name,'dpkg-shlibdeps-params')
         if dpkg_shlibdeps_params:
             need_custom_binary_target = True
-            self.dh_binary_lines = """\tdh binary --before dh_shlibdeps
+            self.dh_binary_arch_lines = """\tdh binary-arch --before dh_shlibdeps
 \tdh_shlibdeps -a --dpkg-shlibdeps-params=%s
 \tdh binary --after dh_shlibdeps"""%dpkg_shlibdeps_params
-
         else:
-            self.dh_binary_lines = '\tdh binary'
+            self.dh_binary_arch_lines = '\tdh binary-arch'
+        self.dh_binary_indep_lines = '\tdh binary-indep'
 
         conflicts = parse_vals(cfg,module_name,'Conflicts')
         provides = parse_vals(cfg,module_name,'Provides')
@@ -913,7 +978,15 @@ XB-Python-Version: ${python:Versions}
         self.udev_rules = parse_val(cfg,module_name,'Udev-Rules')
 
         if need_custom_binary_target:
-            self.binary_target_lines = RULES_BINARY_TARGET%self.__dict__
+            if self.architecture == 'all':
+                self.binary_target_lines = ( \
+                    RULES_BINARY_ALL_TARGET%self.__dict__ + \
+                    RULES_BINARY_INDEP_TARGET%self.__dict__ )
+            else:
+                self.binary_target_lines = ( \
+                    RULES_BINARY_TARGET%self.__dict__ + \
+                    RULES_BINARY_INDEP_TARGET%self.__dict__ + \
+                    RULES_BINARY_ARCH_TARGET%self.__dict__ )
         else:
             self.binary_target_lines = ''
 
@@ -962,6 +1035,7 @@ def build_dsc(debinfo,
               orig_sdist=None,
               patch_posix=0,
               remove_expanded_source_dir=0,
+              debian_dir_only=False,
               ):
     """make debian source package"""
     #    A. Find new dirname and delete any pre-existing contents
@@ -970,7 +1044,10 @@ def build_dsc(debinfo,
 
     # the location of the copied original source package (it was
     # re-recreated in dist_dir)
-    fullpath_repackaged_dirname = os.path.join(dist_dir,repackaged_dirname)
+    if debian_dir_only:
+        fullpath_repackaged_dirname = os.path.abspath(os.curdir)
+    else:
+        fullpath_repackaged_dirname = os.path.join(dist_dir,repackaged_dirname)
 
     ###############################################
     # 1. make temporary original source tarball
@@ -985,25 +1062,26 @@ def build_dsc(debinfo,
     # of "python setup.py sdist" (done above) so that we avoid
     # packaging .svn directories, for example.
 
-    repackaged_orig_tarball = ('%(source)s_%(upstream_version)s.orig.tar.gz'%
-                               debinfo.__dict__)
-    repackaged_orig_tarball_path = os.path.join(dist_dir,
-                                                repackaged_orig_tarball)
-    if orig_sdist is not None:
-        if os.path.exists(repackaged_orig_tarball_path):
-            os.unlink(repackaged_orig_tarball_path)
-        link_func(orig_sdist,repackaged_orig_tarball_path)
-    else:
-        make_tarball(repackaged_orig_tarball,
-                     repackaged_dirname,
-                     cwd=dist_dir)
+    if not debian_dir_only:
+        repackaged_orig_tarball = ('%(source)s_%(upstream_version)s.orig.tar.gz'%
+                                   debinfo.__dict__)
+        repackaged_orig_tarball_path = os.path.join(dist_dir,
+                                                    repackaged_orig_tarball)
+        if orig_sdist is not None:
+            if os.path.exists(repackaged_orig_tarball_path):
+                os.unlink(repackaged_orig_tarball_path)
+            link_func(orig_sdist,repackaged_orig_tarball_path)
+        else:
+            make_tarball(repackaged_orig_tarball,
+                         repackaged_dirname,
+                         cwd=dist_dir)
 
-    # apply patch
-    if debinfo.patch_file != '':
-        apply_patch(debinfo.patch_file,
-                    posix=patch_posix,
-                    level=debinfo.patch_level,
-                    cwd=fullpath_repackaged_dirname)
+        # apply patch
+        if debinfo.patch_file != '':
+            apply_patch(debinfo.patch_file,
+                        posix=patch_posix,
+                        level=debinfo.patch_level,
+                        cwd=fullpath_repackaged_dirname)
 
     for fname in ['Makefile','makefile']:
         if os.path.exists(os.path.join(fullpath_repackaged_dirname,fname)):
@@ -1111,6 +1189,15 @@ def build_dsc(debinfo,
         link_func(fname,
                   os.path.join(debian_dir,'%s.udev'%debinfo.package))
 
+    #    J. debian/source/format
+    os.mkdir(os.path.join(debian_dir,'source'))
+    fd = open( os.path.join(debian_dir,'source','format'), mode='w')
+    fd.write('1.0\n')
+    fd.close()
+
+    if debian_dir_only:
+        return
+
     ###############################################
     # 3. unpack original source tarball
 
@@ -1202,7 +1289,7 @@ Maintainer: %(maintainer)s
 %(uploaders)sSection: %(debian_section)s
 Priority: optional
 Build-Depends: %(build_depends)s
-Standards-Version: 3.7.2
+Standards-Version: 3.8.4
 %(source_stanza_extras)s
 Package: %(package)s
 Architecture: %(architecture)s
@@ -1234,10 +1321,23 @@ unexport LDFLAGS
 """
 
 RULES_BINARY_TARGET = """
-binary: build
-%(dh_binary_lines)s
-%(dh_installmime_line)s
-%(dh_desktop_line)s
+binary: binary-arch binary-indep
+"""
+
+RULES_BINARY_ALL_TARGET = """
+binary: binary-indep
+"""
+
+RULES_BINARY_ARCH_TARGET = """
+binary-arch: build
+%(dh_binary_arch_lines)s
+"""
+
+RULES_BINARY_INDEP_TARGET = """
+binary-indep: build
+%(dh_binary_indep_lines)s
+%(dh_installmime_indep_line)s
+%(dh_desktop_indep_line)s
 """
 
 PREINST = """#! /bin/sh
